use core::fmt;

use crate::query::ops::Operable;

use super::{basic_fn, qry_fn, QryFunc};

/// The avg_over_time rollup query function
#[inline]
pub fn avg_over_time(
    range_vec: impl Operable + 'static,
) -> QryFunc<impl Fn(&mut fmt::Formatter) -> fmt::Result> {
    basic_fn("avg_over_time", range_vec)
}

/// The min_over_time rollup query function
#[inline]
pub fn min_over_time(
    range_vec: impl Operable + 'static,
) -> QryFunc<impl Fn(&mut fmt::Formatter) -> fmt::Result> {
    basic_fn("min_over_time", range_vec)
}

/// The max_over_time rollup query function
#[inline]
pub fn max_over_time(
    range_vec: impl Operable + 'static,
) -> QryFunc<impl Fn(&mut fmt::Formatter) -> fmt::Result> {
    basic_fn("max_over_time", range_vec)
}

/// The sum_over_time rollup query function
#[inline]
pub fn sum_over_time(
    range_vec: impl Operable + 'static,
) -> QryFunc<impl Fn(&mut fmt::Formatter) -> fmt::Result> {
    basic_fn("sum_over_time", range_vec)
}

/// The count_over_time rollup query function
#[inline]
pub fn count_over_time(
    range_vec: impl Operable + 'static,
) -> QryFunc<impl Fn(&mut fmt::Formatter) -> fmt::Result> {
    basic_fn("count_over_time", range_vec)
}

/// The quantile_over_time rollup query function (phi value must be >= 0 and <= 1)
#[inline]
pub fn quantile_over_time(
    phi: f32,
    range_vec: impl Operable + 'static,
) -> QryFunc<impl Fn(&mut fmt::Formatter) -> fmt::Result> {
    qry_fn!(quantile_over_time, "{phi},{range_vec}")
}

/// The stddev_over_time rollup query function
#[inline]
pub fn stddev_over_time(
    range_vec: impl Operable + 'static,
) -> QryFunc<impl Fn(&mut fmt::Formatter) -> fmt::Result> {
    basic_fn("stddev_over_time", range_vec)
}

/// The stdvar_over_time rollup query function
#[inline]
pub fn stdvar_over_time(
    range_vec: impl Operable + 'static,
) -> QryFunc<impl Fn(&mut fmt::Formatter) -> fmt::Result> {
    basic_fn("stdvar_over_time", range_vec)
}

/// The present_over_time rollup query function
#[inline]
pub fn present_over_time(
    range_vec: impl Operable + 'static,
) -> QryFunc<impl Fn(&mut fmt::Formatter) -> fmt::Result> {
    basic_fn("present_over_time", range_vec)
}

/// The last_over_time rollup query function
#[inline]
pub fn last_over_time(
    range_vec: impl Operable + 'static,
) -> QryFunc<impl Fn(&mut fmt::Formatter) -> fmt::Result> {
    basic_fn("last_over_time", range_vec)
}

#[cfg(test)]
mod tests {
    use crate::query::{Duration, Metric, Unit};

    use super::*;

    fn test_metric() -> Metric<'static> {
        Metric::new("test_metric").lookback(Duration(15, Unit::Day))
    }

    #[test]
    fn rollup_avg() {
        let string = avg_over_time(test_metric()).to_string();
        assert_eq!(string, "avg_over_time(test_metric[15d])");
    }

    #[test]
    fn rollup_min() {
        let string = min_over_time(test_metric()).to_string();
        assert_eq!(string, "min_over_time(test_metric[15d])");
    }

    #[test]
    fn rollup_max() {
        let string = max_over_time(test_metric()).to_string();
        assert_eq!(string, "max_over_time(test_metric[15d])");
    }
    #[test]
    fn rollup_sum() {
        let string = sum_over_time(test_metric()).to_string();
        assert_eq!(string, "sum_over_time(test_metric[15d])");
    }
    #[test]
    fn rollup_count() {
        let string = count_over_time(test_metric()).to_string();
        assert_eq!(string, "count_over_time(test_metric[15d])");
    }

    #[test]
    fn rollup_quantile() {
        let string = quantile_over_time(0.1, test_metric()).to_string();
        assert_eq!(string, "quantile_over_time(0.1,test_metric[15d])");
    }
    #[test]
    fn rollup_stddev() {
        let string = stddev_over_time(test_metric()).to_string();
        assert_eq!(string, "stddev_over_time(test_metric[15d])");
    }

    #[test]
    fn rollup_stdvar() {
        let string = stdvar_over_time(test_metric()).to_string();
        assert_eq!(string, "stdvar_over_time(test_metric[15d])");
    }
    #[test]
    fn rollup_present() {
        let string = present_over_time(test_metric()).to_string();
        assert_eq!(string, "present_over_time(test_metric[15d])");
    }

    #[test]
    fn rollup_last() {
        let string = last_over_time(test_metric()).to_string();
        assert_eq!(string, "last_over_time(test_metric[15d])");
    }
}
